/*
 * $Id: ExecutableChannels.java,v 1.2 2004/09/19 10:33:04 kredel Exp $
 */

//package edu.jas.util;
package comm;

import util.Logger;

import java.io.IOException;
import java.io.FileNotFoundException;
//import java.io.Reader;
import java.io.FileReader;
import java.io.BufferedReader;

//import java.util.Iterator;
import java.util.List;
import java.util.ArrayList;

//import org.apache.log4j.Logger;

//import edu.unima.ky.parallel.ChannelFactory;
//import edu.unima.ky.parallel.SocketChannel;

/**
 * Class ExecutableChannels
 * used to receive and execute classes
 * @author Heinz Kredel
 */


public class ExecutableChannels {

    // private static Logger logger = Logger.getLogger(ExecutableChannels.class);
    private static Logger logger = new Logger();

    protected final ChannelFactory cf;
    protected SocketChannel[] channels = null;
    protected String[] servers = null;
    protected int[] ports = null;
    protected final int DEFAULT_PORT = ChannelFactory.DEFAULT_PORT;


    protected ExecutableChannels() {
        cf = new ChannelFactory();
    }


    /**
     * Constructor from array of server:port strings
     */

    public ExecutableChannels(String[] srvs) {
        this();
        if ( srvs == null ) {
            return;
        }
        servers = new String[ srvs.length ];
        ports = new int[ srvs.length ];
        for ( int i = 0; i < srvs.length; i++ ) {
            setServerPort( i, srvs[i] );
        }
    }


    /**
     * Constructor from maschine file
     */

    public ExecutableChannels(String fname) throws FileNotFoundException {
        this();
        BufferedReader in = new BufferedReader( new FileReader( fname ) );
        String line = null;
        List list = new ArrayList();
	int x;
        try {
            while (true) {
               if ( !in.ready() ) {
                  break;
               }
               line = in.readLine();
               x = line.indexOf("#");
               if ( x >= 0 ) {
                   line = line.substring(0,x);
               }
               line = line.trim();
               if ( line.length() == 0 ) {
                   continue;
               }
               list.add(line);
            }
        } catch (IOException e) {
        }
        logger.debug("list.size() in " + fname + " = " + list.size());
        if ( list.size() == 0 ) {
            return;
        }
        servers = new String[ list.size() ];
        ports = new int[ list.size() ];
        for ( int i = 0; i < servers.length; i++ ) {
            setServerPort( i, (String)list.get( i ) );
        }
    }


    /* 
     * internal method
     */
    protected void setServerPort(int i, String srv) {
        int x = srv.indexOf(":");
        ports[i] = DEFAULT_PORT;
        if ( x < 0 ) {
           servers[i] = srv;
        } else {
           servers[i] = srv.substring(0,x);
           String p = srv.substring(x+1,srv.length());
           try { 
                ports[i] = Integer.parseInt( p );
           } catch (NumberFormatException ignored) {
           } 
        }
    }


/**
 * String representation
 */ 
    public String toString() {
        StringBuffer s = new StringBuffer("ExecutableChannels(");
        if ( servers != null ) {
           for ( int i = 0; i < servers.length; i++ ) {
               s.append( servers[i] + ":" + ports[i] );
               if ( i < servers.length-1 ) {
                  s.append(" ");
               }
           }
        }
        s.append(")");
        return s.toString();
    }


/**
 * number of servers
 */ 
    public int numServers() {
        if ( servers != null ) {
           return servers.length;
        } else {
            return -1;
        }
    }

/**
 * get master host
 */ 
    public String getMasterHost() {
        if ( servers != null && servers.length > 0 ) {
           return servers[0];
        } else {
            return null;
        }
    }

/**
 * get master port
 */ 
    public int getMasterPort() {
        if ( ports != null && ports.length > 0 ) {
           return ports[0];
        } else {
            return 0;
        }
    }


/**
 * number of channels
 */ 
    public int numChannels() {
        if ( channels != null ) {
           return channels.length;
        } else {
            return -1;
        }
    }


/**
 * open, setup of SocketChannels 
 */ 
    public void open() throws IOException {
        logger.debug("opening " + servers.length + " channels");
	if ( servers.length <= 1 ) {
	    throw new IOException("to few servers");
	}
        channels = new SocketChannel[ servers.length-1 ];
        for ( int i = 1; i < servers.length; i++ ) {
            channels[i-1] = cf.getChannel( servers[i], ports[i] );
        }
    }


/**
 * open, setup of SocketChannels 
 */ 
    public void open(int nc) throws IOException {
        logger.debug("opening " + nc + " channels");
	if ( servers.length <= 1 ) {
	    throw new IOException("to few servers");
	}
        channels = new SocketChannel[ nc ];
        int j = 1; // 0 is master
        for ( int i = 0; i < channels.length; i++ ) {
            if ( j >= servers.length ) { // modulo #servers
                j = 1;
            }
            channels[i] = cf.getChannel( servers[j], ports[j] );
            j++;
       }
    }


/**
 * close all channels and ChannelFactory
 */ 

    public void close() {
        logger.debug("closing ExecutableChannels");
        if ( cf != null ) cf.terminate();
        if ( channels != null ) {
            for ( int i = 0; i < channels.length; i++ ) {
               if ( channels[i] != null ) {
                  channels[i].close();
		  channels[i] = null;
               }
           }
           channels = null;
        }
        logger.debug("ExecuteChannels closed");
    }


/**
 * getChannel
 * @param i channel number
 */ 
    public SocketChannel getChannel(int i) {
        if ( channels != null && 0 <= i && i < channels.length ) {
           return channels[i];
        } else {
            return null;
        }
    }


/**
 * getChannels
 */ 
    public SocketChannel[] getChannels() {
        return channels;
    }


/**
 * send on channel i
 * @param i channel number
 * @param o object to send
 */ 
    public void send(int i, Object o) throws IOException {
        if ( channels != null && 0 <= i && i < channels.length ) {
           channels[i].send(o);
        } 
    }


/**
 * recieve on channel i
 * @param i channel number
 * @return object recieved
 */ 
    public Object receive(int i) throws IOException, ClassNotFoundException {
        if ( channels != null && 0 <= i && i < channels.length ) {
           return channels[i].receive();
        } else {
	    return null;
	}

    }

}
